from roboflow import Project
import roboflow
from roboflow.core.version import Version
import os
from typing import Optional, List
from rf100vl.util import get_basename, get_category
import json
from copy import deepcopy


class NotDownloadedError(Exception):
    pass


class RF100VlDataset:
    def __init__(self, rf_project: Project):
        self.rf_project: Project = rf_project
        self.category: str = get_category(get_basename(rf_project.name))
        self.name: str = get_basename(rf_project.name)
        self._location: str = None

    def __repr__(self):
        return f"RF100VLDataset(name={self.name}, category={self.category})"

    def download(self, path: str, model_format: str = "coco", overwrite: bool = True):
        os.makedirs(path, exist_ok=True)
        versions: List[Version] = self.rf_project.versions()
        latest_version: Version = max(versions, key=lambda v: v.id)
        dataset =latest_version.download(
            location=path, model_format=model_format, overwrite=overwrite
        )
        if model_format == "coco":
            self.clean_coco_dataset(path)
        
        self._location = dataset.location

    @property
    def is_fewshot(self) -> str:
        return self.name.contains("fsod")

    @property
    def location(self) -> Optional[str]:
        if self._location is None:
            raise NotDownloadedError("Dataset has not been downloaded yet")
        return self._location

    def clean_coco_dataset(self, path: str) -> None:
        """
        Get clean annotation data: Removes class 0 that is added by default to Roboflow datasets and shifts annotation ids by 1
        """
        for split in ["train", "test", "valid"]:
            anno_path = os.path.join(path, split, "_annotations.coco.json")
            if not os.path.exists(anno_path):
                continue
            with open(anno_path, "r") as f:
                data_ann = json.load(f)
            clean_ann_data = self.get_clean_ann_data(data_ann)
            with open(anno_path, "w") as f:
                json.dump(clean_ann_data, f)

    def get_clean_ann_data(self, data_ann):
        new_data_ann = {}
        if data_ann["info"]:
            new_data_ann["info"] = data_ann["info"]
        if data_ann["licenses"]:
            new_data_ann["licenses"] = data_ann["licenses"]

        # confirm if category 0 is none
        if data_ann["categories"][0]["supercategory"] != "none":
            return

        new_data_ann["categories"] = [
            {
                "id": cat["id"] - 1,
                "name": cat["name"],
                "supercategory": cat["supercategory"],
            }
            for cat in data_ann["categories"]
            if cat["id"] != 0
        ]

        new_data_ann["images"] = data_ann["images"]
        new_data_ann["annotations"] = deepcopy(data_ann["annotations"])

        for ann in new_data_ann["annotations"]:
            ann["category_id"] = ann["category_id"] - 1

        return new_data_ann



import os
from typing import List, Optional, Iterator
from rf100vl.dataset import RF100VlDataset
from roboflow import Project
import roboflow

def get_rf(api_key: Optional[str] = None):
    if api_key is not None:
        return roboflow.Roboflow(api_key=api_key)
    else:
        ROBOFLOW_API_KEY = os.getenv("ROBOFLOW_API_KEY")
        if ROBOFLOW_API_KEY is not None:
            return roboflow.Roboflow(api_key=ROBOFLOW_API_KEY)
        else:
            roboflow.login()
            return roboflow.Roboflow()


class DatasetList:
    def __init__(self, projects: List[Project]):
        self.projects = projects
        self.projects = sorted(self.projects, key=lambda p: p.name)
        self.datasets = [RF100VlDataset(p) for p in projects]

    def __iter__(self) -> Iterator[RF100VlDataset]:
        return iter(self.datasets)

    def download(self, path: str, model_format: str = "coco", overwrite: bool = True) -> None:
        os.makedirs(path, exist_ok=True)
        for dataset in self.datasets:
            dataset.download(os.path.join(path, dataset.name), model_format, overwrite)
    
    def __len__(self) -> int:
        return len(self.datasets)
    
    def __getitem__(self, index: int) -> RF100VlDataset:
        return self.datasets[index]

def get_rf100vl_projects(api_key: Optional[str] = None):
    rf = get_rf(api_key)
    workspace = rf.workspace("rf100-vl")
    projects = []
    for project in workspace.project_list:
        project = Project(api_key=rf.api_key, a_project=project, model_format="coco")
        projects.append(project)

    return DatasetList(projects)


def get_rf100vl_fsod_projects(api_key: Optional[str] = None):
    rf = get_rf(api_key)
    workspace = rf.workspace("rf100-vl-fsod")
    projects = []
    for project in workspace.project_list:
        project = Project(api_key=api_key, a_project=project, model_format="coco")
        projects.append(project)
    return DatasetList(projects)


def get_rf20vl_fsod_projects(api_key: Optional[str] = None) -> DatasetList:
    rf = get_rf(api_key)
    workspace = rf.workspace("rf20-vl-fsod")
    projects = []
    for project in workspace.project_list:
        project = Project(api_key=api_key, a_project=project, model_format="coco")
        projects.append(project)
    return DatasetList(projects)


def get_rf20vl_full_projects(api_key: Optional[str] = None) -> DatasetList:
    rf = get_rf(api_key)
    workspace = rf.workspace("rf20-vl")
    projects = []
    for project in workspace.project_list:
        project = Project(api_key=api_key, a_project=project, model_format="coco")
        projects.append(project)
    return DatasetList(projects)


def download_rf100vl(path: str, model_format: str = "coco", overwrite: bool = True, api_key: Optional[str] = None) -> DatasetList:
    rf100vl_projects = get_rf100vl_projects(api_key)
    rf100vl_projects.download(path, model_format, overwrite)
    return rf100vl_projects


def download_rf100vl_fsod(
    path: str, model_format: str = "coco", overwrite: bool = True, api_key: Optional[str] = None
) -> DatasetList:
    rf100vl_fsod_projects = get_rf100vl_fsod_projects(api_key)
    rf100vl_fsod_projects.download(path, model_format, overwrite)
    return rf100vl_fsod_projects


def download_rf20vl_fsod(path: str, model_format: str = "coco", overwrite: bool = True, api_key: Optional[str] = None) -> DatasetList:
    rf20vl_fsod_projects = get_rf20vl_fsod_projects(api_key)
    rf20vl_fsod_projects.download(path, model_format, overwrite)
    return rf20vl_fsod_projects


def download_rf20vl_full(path: str, model_format: str = "coco", overwrite: bool = True, api_key: Optional[str] = None) -> DatasetList:
    rf20vl_full_projects = get_rf20vl_full_projects(api_key)
    rf20vl_full_projects.download(path, model_format, overwrite)
    return rf20vl_full_projects


import json
import os

DATASET_TO_CATEGORY_JSON_PATH = os.path.join(os.path.dirname(__file__), "assets", "dataset_name_to_category.json")
DATASET_TO_BASENAME_JSON_PATH = os.path.join(os.path.dirname(__file__), "assets", "dataset_name_to_basename.json")

def get_category_json():
    with open(DATASET_TO_CATEGORY_JSON_PATH, "r") as f:
        return json.load(f)
    
def get_basename_json():
    with open(DATASET_TO_BASENAME_JSON_PATH, "r") as f:
        dataset_to_basename = json.load(f)
    
    assert len(set(dataset_to_basename.values())) == 100
    return dataset_to_basename

DATASET_TO_CATEGORY_JSON = get_category_json()
DATASET_TO_BASENAME_JSON = get_basename_json()
keys = list(sorted(set(DATASET_TO_CATEGORY_JSON.keys())))
values = list(sorted(set(DATASET_TO_BASENAME_JSON.values())))
assert set(DATASET_TO_CATEGORY_JSON.keys()) == set(DATASET_TO_BASENAME_JSON.values())
assert len(keys) == len(values) == 100

def get_category(dataset_name: str):
    return DATASET_TO_CATEGORY_JSON[dataset_name]

def get_basename(dataset_name: str):
    return DATASET_TO_BASENAME_JSON[dataset_name]



from setuptools import setup, find_packages

# Read requirements
with open('requirements.txt') as f:
    requirements = f.read().splitlines()

# Read README
with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="rf100vl",
    version="1.0.0",
    description="RF100-VL Dataset Interface",
    author="Roboflow, Inc.",
    author_email="peter@roboflow.com",
    packages=find_packages(),
    python_requires=">=3.9",
    install_requires=requirements,
    long_description=long_description,
    long_description_content_type="text/markdown",
    include_package_data=True,
    package_data={
        "rf100vl": ["assets/*.json"],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    keywords="machine-learning, deep-learning, computer-vision, ML, DL, CV, AI"
)


from rf100vl.roboflow100vl import (
    get_rf100vl_fsod_projects,
    get_rf100vl_projects,
    get_rf20vl_fsod_projects,
    get_rf20vl_full_projects,
    download_rf100vl_fsod,
    download_rf100vl,
    download_rf20vl_fsod,
    download_rf20vl_full,
)
